<?php
namespace Noalyss\Forms;
require_once NOALYSS_INCLUDE."/lib/ac_common.php";
/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// Copyright Author Dany De Bontridder danydb@aevalys.eu 31/10/23
/*! 
 * \file
 * \brief record the input from the web page
 */

/*!
 * \class
 * \brief record the input from the web page
 */

class Data_Record
{
    protected $data_received_raw; //<! $data received_raw Data_received_raw_SQL
    private $error; //!< $error string error
    function __construct()
    {
        $this->data_received_raw=null;
        $this->error=null;
    }

    /**
     * @return null
     */
    public function get_data_received_raw():Data_received_raw_SQL
    {
        return $this->data_received_raw;
    }

    /**
     * @param null $this->data_received_raw
     */
    public function set_data_received_raw(Data_received_raw_SQL $data_received_raw): Data_Record
    {
        $this->data_received_raw = $data_received_raw;
        return $this;
    }

    /**
     * @brief action to do before committing, could be implemented. If parameters are needed must use
     * @return bool true if succedd , false otherwise
     */
    protected function precommit():bool
    {
        return true;
    }

    /**
     * @brief action to do after committing , could be implemented
     * @return bool true if succedd , false otherwise
     */
    protected function postcommit():bool
    {
        return true;
    }

    /**
     * @brief action to take if precommit returns false
     * @return void
     */
    protected function error_precommit() {

    }

    /**
     * @brief action to take if postcommit returns false
     * @return void
     */
    protected function error_postcommit() {

    }
    /**
     * @brief find the corresponding form and record it, the form action can be a post or get
     * @code

    // Example of page receiving the data

    require_once '/home/dany/developpement/phpcompta/noalyss/include/constant.php';
    require_once '/home/dany/developpement/phpcompta/noalyss/include/ext/noalyss-form/nform_constant_min.php';

    \Noalyss\Forms\Data_Record::record();
     *
     * @endcode
     * @param $p_method = POST , GET or REQUEST , way to take data
     * @param $resend = 'all' make an URL with the parameter and add them to the url of the answer
     */
    public function record($p_method='REQUEST',$resend='all')
    {
        $http=new \HttpInput();

        try {
            if ( ! in_array($p_method,['REQUEST','POST','GET'] )) {
                throw new \Exception('DR55'."method [$p_method] invalide: possible options are POST, GET or REQUEST");
            }
            $cn=\Dossier::connect();
            if ($p_method == "GET")            $token=$http->get("token");
            if ($p_method == "POST")            $token=$http->post("token");
            if ($p_method == "REQUEST")            $token=$http->request("token");
            $dr_from=$_SERVER['REMOTE_ADDR']??'';
            if ($cn->get_value('select count(*) from noalyss_form.form where 
                                           f_token=$1 and (f_url_allow is null or f_url_allow=$2)'
                    ,[$token,$dr_from])==0) {
                throw new \Exception ("Unknown request token $token from $dr_from");
            }
        } catch (\Exception $e) {
            \record_log($e->getMessage());
            throw $e;
        }

        try {
            $form_id=$cn->get_value("select f_id from noalyss_form.form where f_token=$1",[$token]);
            $cn->start();
            $this->data_received_raw=new Data_received_raw_SQL($cn);
            $this->data_received_raw->setp('dr_from',$dr_from);
            $this->data_received_raw->insert();
            $form_detail_sql=new Form_detail_SQL($cn);
            $a_form_detail_sql=$form_detail_sql->collect_objects ('where form_id=$1',[$form_id]);
\Noalyss\Dbg::echo_var(1, '<hr>');
            foreach ($a_form_detail_sql as $form_detail_sql) {
                $parm=$form_detail_sql->getp('fd_post_param');
\Noalyss\Dbg::echo_var(1, "param [$parm]");
\Noalyss\Dbg::echo_var(1, "method [$p_method]");

                if ($p_method == "GET")            $value=$http->get($parm,'string','');
                if ($p_method == "POST")            $value=$http->post($parm,'string','');
                if ($p_method == "REQUEST")            $value=$http->request($parm,'string','');

                if ($form_detail_sql->getp('fd_optional')==1 && $value=="") {
                    throw new \Exception('err:data_record:148 empty '.$parm);
                }
\Noalyss\Dbg::echo_var(1, "value [$value]");
\Noalyss\Dbg::echo_var(1, '<hr>');

                $data_received_raw_detail_sql=new Data_received_raw_detail_SQL($cn);
                $data_received_raw_detail_sql->setp('form_detail_id',$form_detail_sql->getp('fd_id'));
                $data_received_raw_detail_sql->setp('drd_post_value',$value);
                $data_received_raw_detail_sql->setp('data_received_raw_id',$this->data_received_raw->dr_id);
                $data_received_raw_detail_sql->insert();
            }
            if ( $this->precommit() == false ) {
                $this->error_precommit();
            }

            if ( $this->check_attribut() == false ) {
                throw new \Exception('err:data_record:153'.$this->error);
            }
            $cn->commit();
            if ( $this->postcommit() == false ) {
                $this->error_postcommit();
            }

            $url_redirect =$cn->get_value("select f_url_success from noalyss_form.form where f_id=$1",[$form_id]);
            if ( ! empty ($url_redirect)) {
                $url=($resend == 'all')?http_build_query($_REQUEST):'';
                $url_redirect=Noalyss_Form::url_add_param($url_redirect,$url);
                Noalyss_Form::redirect($url_redirect);
            }
        } catch (\Exception $e) {
            $cn->rollback();
            \record_log($e->getMessage());
            if (DEBUGNOALYSS > 0 ) echo $e->getMessage();
            $url_redirect =$cn->get_value("select f_url_success from noalyss_form.form where f_id=$1",[$form_id]);
            if ( ! empty ($url_redirect)) {
                $url=($resend == 'all')?http_build_query($_REQUEST):'';
                $url_redirect=Noalyss_Form::url_add_param($url_redirect,$url);
                Noalyss_Form::redirect($url_redirect);
            } else {
                throw new \Exception($e);
            }

        }
    }

    /**
     * @brief check that the record is correct with the AD_ID type
     * @return boolean
     */
    function check_attribut():bool
    {
        $cn=\Dossier::connect();

        $data_received_raw_id=$this->data_received_raw->get('dr_id');
        $a_drd_id = $cn->get_array("select drd_id,drd_post_value,form_detail_id,fd.ad_id,ad1.ad_type,ad1.ad_extra 
            from noalyss_form.data_received_raw_detail 
            join noalyss_form.form_detail fd on fd.fd_id = data_received_raw_detail.form_detail_id
            join attr_def ad1 on (ad1.ad_id=fd.ad_id)
    where data_received_raw_id=$1
            ", [$data_received_raw_id]);

        foreach ($a_drd_id as $item)
        {

            $value=strip_tags(trim($item['drd_post_value']??""));


            switch ($item['ad_type']){
                case 'text':
                case 'zone':
                if ( $value == "") $cn->exec_sql('update noalyss_form.data_received_raw_detail set drd_post_value=$2 where drd_id = $1 ',
                    [$item['drd_id'],$value]);

                break;
                case 'card':
                    // quick code of a card
                    if ( $value == "") break;
                    if ( $cn->get_value('select count(*) from fiche_detail where ad_id=$1 and upper(ad_value)=upper($2)',
                        [ATTR_DEF_QUICKCODE,$value]) == 0
                    ){
                        $this->error=sprintf("drd_id [%s] QCODE [%s] is invalid value[%s]",$item['drd_id'],$item['ad_id'],$value);
                        return false;
                    };
                    break;
                case 'poste':
                    if ( $value == "") break;
                    // check that accounting exists
                    if ( $cn->get_value('select count(*) from tmp_pcmn where pcm_val=$1',
                            [$value]) == 0
                    ){
                        $this->error=sprintf("drd_id [%s] ACCOUNTING [%s] is invalid value[%s]",$item['drd_id'],$item['ad_id'],$value);
                        return false;
                    };
                    break;
                case 'numeric':
                    if ( $value == "")
                    {
                        $cn->exec_sql('update noalyss_form.data_received_raw_detail set drd_post_value=0 where drd_id = $1 ',
                            [$item['drd_id']]);
                        break;
                    }
                    // must be a number or 0 if empty
                    if (isNumber($value)==0) {
                        $this->error=sprintf("drd_id [%s] AD_ID [%s] has an invalid value [%s]",$item['drd_id'],$item['ad_id'],$value);
                        return false;
                    }
                    break;
                case 'date':
                    if ( $value == "") break;
                    $from_date=\DateTime::createFromFormat('d.m.Y',$value);
                    if ( $from_date == false || $from_date->format('d.m.Y') != $value) {

                        $this->error=sprintf("drd_id [%s] DATE [%s] has an invalid value [%s]",$item['drd_id'],$item['ad_id'],$value);
                        return false;
                    }
                    break;
                case 'check':
                    if ( $value == "") {
                        $cn->exec_sql('update noalyss_form.data_received_raw_detail set drd_post_value=0 where drd_id = $1 ',
                            [$item['drd_id']]);
                        break;
                    }
                    if ( $value != 1 && $value != 0 ){
                        $this->error=sprintf("drd_id [%s] CHECK [%s] is invalid value[%s]",$item['drd_id'],$item['ad_id'],$value);
                        return false;
                    }
                    break;
                case 'select':
                    if ( $value == "") break;
                    $array=$cn->get_array($item['ad_extra']);
                    $aValid=array_column($array,'?column?');
                    if ( in_array($value, $aValid) == false) {
                        $this->error=sprintf("drd_id [%s] SELECT [%s] is invalid value[%s]",$item['drd_id'],$item['ad_id'],$value);
                        return false;
                    }



                    break;
            }

        }
        return true;
    }

    /**
     * @brief insert a card
     * @throws \Exception
     */
    function insert_card()
    {
        try {
            $cn=\Dossier::connect();
            /**
             * var $data_received_raw_id int  pk of NOALYSS_FORM.DATA_RECEIVED_RAW (dr_id)
             */
            $data_received_raw_id=$this->data_received_raw->get('dr_id');
            $a_drd_id = $cn->get_array("select drd_id,drd_post_value,form_detail_id,ad_id 
            from noalyss_form.data_received_raw_detail 
            join noalyss_form.form_detail fd on fd.fd_id = data_received_raw_detail.form_detail_id
    where data_received_raw_id=$1
            ", [$data_received_raw_id]);
            $cn->start();
            $cat = -1;
            foreach ($a_drd_id as $drd_id) {
                // build the card
                $array['av_text' . $drd_id['ad_id']] = $drd_id['drd_post_value'];
                if ( $cat == -1 ) {
                    $cat=$cn->get_value("select fiche_def_id from 
                            noalyss_form.form f1 
                                join noalyss_form.form_detail fd1 on f1.f_id = fd1.form_id
                                where fd1.fd_id = $1",[$drd_id['form_detail_id']]);
                }
            }
            if ( $cat == - 1||$cat == "") {
                throw new \Exception ("err:ASR82. Aucune catégorie");
            }
            $card=new \Fiche($cn);

            $card->insert($cat,$array);
            $cn->exec_sql("update noalyss_form.data_received_raw set dr_status=1, fiche_id=$2 where dr_id=$1",[$data_received_raw_id,$card->id]);
            $cn->commit();

        } catch (\Exception $e) {
            \record_log($e);
            throw $e;
        }

    }
}
